# Makefile for Pointless, Othercell & associate testing
# Note:
#  use -O not -O2
#  -static doesn't work on hex

# Compilers
CC = cc
CXX=c++
#CXX=icc

# Compiler flags
#OPT = -g -m32
#OPT = -g 
#OPT = -O -m32 -Wall
OPT = -O -static
#OPT = -O0 -g3

CXXFLAGS=  ${OPT}
CFLAGS= ${OPT}
LFLAGS= ${OPT}


# CCTBX locations
CCTBX=${CLIB}/cctbx
#CCTBX=/usr/local/CCTBX
CCTBX_sources=${CCTBX}/cctbx_sources
#  Version 1 = 2006
#CCTBX_VERSION=2006
#  Version 2 = 2007
CCTBX_VERSION=2007

XCPPFLAGS=  -DCCTBX_VERSION=${CCTBX_VERSION}
#XCPPFLAGS= -ftemplate-depth-120  -Wno-long-double
#XCPPFLAGS= -ftemplate-depth-120 -fcoalesce-templates -Wno-long-double

# Clipper location
CLPLIB=${CLIB}
CLPR=${CLIB}/clipper

#<<<<< Includes
# CCP4
ICCP4=-I${CLIBS}

# Clipper
#ICLPR=-I${CLPR}/include
ICLPR=-I${CLPR} -I${CLPR}/clipper/core

# cctbx
ITBX=\
-I${CCTBX_sources}/boost \
-I${CCP4}/include \
-I${CCTBX_sources}/cctbx_project
##-I${CCTBX_sources}/cctbx_project/scitbx/include

INCLUDE=-I. ${ITBX} ${ICLPR} ${ICCP4}

#<<<<< Libraries
# CCP4
#LCCP4=
LCCP4=-L${CLIB} -lrfftw -lfftw -lccp4f -lccp4c -lmmdb -lm

# Clipper
LCLPR=-L${CLPLIB} -lclipper-ccp4 -lclipper-contrib -lclipper-minimol -lclipper-mmdb -lclipper-core 

# cctbx
LTBX= ${CLIB}/libcctbx.a
#LTBX= -L${CLIB} -lcctbx 
#LTBX= -L${CCTBX}/cctbx_build/lib  -lcctbx 

SLIB = -Wl,--whole-archive -lpthread -Wl,--no-whole-archive
#SLIB=

LDLIBS=${LTBX} ${LCLPR} ${LCCP4} ${SLIB}


CPPFLAGS=${XCPPFLAGS}  ${INCLUDE}
CCFLAGS = ${INCLUDE} 

TARGETS = pointless othercell
all: ${TARGETS}

####################################################################
#<<<<< Pointless
EXE = pointless
SRC=${EXE}.cpp

# Unique to Pointless
OBJ = \
	InputAll.o average.o centre.o checklatticecentering.o \
	checknullbatches.o choosesolution.o comparestring.o \
	completeness.o copymergedmtz.o datatypes.o expandfilename.o \
	filetype.o fix_merged_scores.o getCCsigfac.o globalcontrols.o \
	intensitystatistics.o io_files.o keywords.o listdir.o \
	makehkl_listscompatible.o normaliseunmerge.o \
	outputunmergeddata.o pgscore.o \
	printthings.o readallhklinfiles.o sca_unmerge.o \
	scsignificance.o setscores.o sysabsstats.o sysabstest.o \
	sysabszones.o writesummedlist.o writeunmergedmtz.o \
	xds_unmerge.o \
	testlauegroup.o testindex.o twintest.o

# Shared with Aimless
COBJ = \
	CCP4base.o Errors.o Output.o Preprocessor.o \
	cctbx_utils.o cellgroup.o checkcompatiblesymmetry.o \
	columnlabels.o controls.o eprob.o fileread.o \
	getsubgroups.o hash.o hkl_controls.o hkl_datatypes.o \
	hkl_merged_list.o hkl_symmetry.o hkl_unmerge.o icering.o \
	interpretcommandline.o jiffy.o latsym.o lattice.o \
	linearlsq.o matvec_utils.o \
	mtz_merge_io.o mtz_unmerge_io.o mtz_utils.o normalise.o \
	numbercomplete.o observationflags.o \
	openinputfile.o pointgroup.o probfunctions.o range.o \
	rotation.o runthings.o scala_util.o score_datatypes.o \
	spacegroupreindex.o spline.o string_util.o timer.o \
	zone.o tablegraph.o


OBJECTS = ${EXE}.o ${OBJ} ${COBJ}

# Unique to Pointless
HDR= \
	InputAll.hh average.hh centre.hh checklatticecentering.hh \
	checknullbatches.hh choosesolution.hh comparestring.hh \
	completeness.hh \
	copymergedmtz.hh datatypes.hh expandfilename.hh filetype.hh \
	fix_merged_scores.hh getCCsigfac.hh globalcontrols.hh \
	intensitystatistics.hh io_files.hh keywords.hh listdir.hh twintest.hh \
	makehkl_listscompatible.hh normaliseunmerge.hh outputunmergeddata.hh \
	pgscore.hh pointless.hh printthings.hh readallhklinfiles.hh \
	reindexscore.hh sca_unmerge.hh scale_model.hh scsignificance.hh \
	setscores.hh spline.hh sysabsstats.hh sysabstest.hh sysabszones.hh \
	version.hh writesummedlist.hh writeunmergedmtz.hh xds_unmerge.hh \
	testlauegroup.hh testindex.hh


# Shared with Aimless
CHDR = \
	CCP4base.hh Errors.hh InputBase.hh Output.hh Preprocessor.hh \
	cctbx_utils.hh cellgroup.hh checkcompatiblesymmetry.hh columnlabels.hh \
	controls.hh eprob.hh fileread.hh getsubgroups.hh hash.hh \
	hkl_controls.hh hkl_datatypes.hh hkl_merged_list.hh hkl_symmetry.hh \
	hkl_unmerge.hh icering.hh interpretcommandline.hh jiffy.hh latsym.hh \
	lattice.hh linearlsq.hh matvec_utils.hh mtz_merge_io.hh \
	mtz_unmerge_io.hh mtz_utils.hh normalise.hh numbercomplete.hh \
	observationflags.hh openinputfile.hh phaser_types.hh pointgroup.hh \
	probfunctions.hh range.hh rotation.hh runthings.hh scala.hh \
	scala_util.hh score_datatypes.hh spacegroupreindex.hh spline.hh \
	string_util.hh timer.hh util.hh zone.hh tablegraph.hh

# Unique to Pointless
SOURCE = \
	${EXE}.cpp \
	InputAll.cpp average.cpp centre.cpp checklatticecentering.cpp \
	checknullbatches.cpp choosesolution.cpp comparestring.cpp \
	completeness.cpp copymergedmtz.cpp datatypes.cpp expandfilename.cpp \
	filetype.cpp fix_merged_scores.cpp getCCsigfac.cpp globalcontrols.cpp \
	intensitystatistics.cpp io_files.cpp keywords.cpp listdir.cpp \
	twintest.cpp makehkl_listscompatible.cpp normaliseunmerge.cpp \
	outputunmergeddata.cpp pgscore.cpp \
	printthings.cpp readallhklinfiles.cpp saint.cpp sca_unmerge.cpp \
	scsignificance.cpp setscores.cpp sysabsstats.cpp sysabstest.cpp \
	sysabszones.cpp writesummedlist.cpp writeunmergedmtz.cpp \
	xds_unmerge.cpp \
	testlauegroup.cpp testindex.cpp


# common with Aimless
CSOURCE = \
	CCP4base.cpp Errors.cpp Output.cpp Preprocessor.cpp \
	cctbx_utils.cpp cellgroup.cpp checkcompatiblesymmetry.cpp \
	columnlabels.cpp controls.cpp eprob.cpp fileread.cpp \
	getsubgroups.cpp hash.cpp hkl_controls.cpp hkl_datatypes.cpp \
	hkl_merged_list.cpp hkl_symmetry.cpp hkl_unmerge.cpp icering.cpp \
	interpretcommandline.cpp jiffy.cpp latsym.cpp lattice.cpp \
	linearlsq.cpp matvec_utils.cpp \
	mtz_merge_io.cpp mtz_unmerge_io.cpp mtz_utils.cpp normalise.cpp \
	numbercomplete.cpp observationflags.cpp \
	openinputfile.cpp pointgroup.cpp probfunctions.cpp range.cpp \
	rotation.cpp runthings.cpp scala_util.cpp score_datatypes.cpp \
	spacegroupreindex.cpp spline.cpp string_util.cpp timer.cpp \
	zone.cpp tablegraph.cpp

DOC = pointless.html

${EXE}: ${OBJECTS} ${HDR}
	${CXX} ${LFLAGS} -o ${EXE} ${OBJECTS}  ${LDLIBS}

####################################################################
OC_OBJ = othercell.o latsym.o hkl_symmetry.o hkl_datatypes.o \
         scala_util.o pointgroup.o pgscore.o \
         scsignificance.o lattice.o matvec_utils.o getsubgroups.o \
	 Output.o zone.o probfunctions.o range.o\
         interpretcommandline.o  rotation.o cctbx_utils.o \
         Preprocessor.o CCP4base.o Errors.o jiffy.o  string_util.o \
	score_datatypes.o  spacegroupreindex.o

OC_SOURCE = othercell.cpp

othercell: ${OC_OBJ}
	${CXX} ${LFLAGS} -o othercell ${OC_OBJ}  ${LDLIBS}

####################################################################
# Export etc

export:
	/bin/rm -rf ../Export/
	mkdir ../Export
	cp ${SOURCE} ../Export/
	cp ${HDR} ../Export/
	cp ${CSOURCE} ../Export/
	cp ${CHDR} ../Export/
	cp ${OC_SOURCE} ../Export/
	cp ${DOC} ../Export/
	cp Makefile ../Export/Makefile.make
	cp Makefile.osx ../Export/Makefile.osx.make

tar: export
	cd ../Export; tar cvf ../pointless.tar .
	cd ..; gzip -f pointless.tar;cp pointless.tar.gz ${scr0}

####################################################################
# Library
POINTLESS_LIB = libpointless.a

library: ${POINTLESS_LIB}

${POINTLESS_LIB}: ${OBJECTS}
	ar -r ${POINTLESS_LIB} ${OBJECTS}
	ranlib ${POINTLESS_LIB}

####################################################################

.SUFFIXES: .cpp .o

.cpp.o: ${HDR}
	${CXX} -c $(CXXFLAGS) $(CPPFLAGS) $<

.c.o:
	${CC} -c $(CFLAGS) $(CCFLAGS) $<

clean: 
	rm *.o ${TARGETS}  ${POINTLESS_LIB} *.gch

####################################################################
#
# Other things (testing etc)

TOBJ=testsym.o \
hkl_symmetry.o hkl_datatypes.o pointgroup.o matvec_utils.o \
latsym.o scala_util.o lattice.o Output.o zone.o getsubgroups.o pgscore.o \
scsignificance.o range.o sysabszones.o rotation.o probfunctions.o jiffy.o \
spacegroupreindex.o

TEXE=testsym

testsym: ${TOBJ}
	${CXX} $(LFLAGS) -o ${TEXE} ${TOBJ}  ${LDLIBS}

TQEXE=test
TQOBJ = test.o score_datatypes.o

#TQOBJ = test.o hkl_datatypes.o matvec_utils.o scala_util.o  range.o  rotation.o  Output.o   jiffy.o
test: ${TQOBJ}
	${CXX} $(LFLAGS) -o ${TQEXE} ${TQOBJ}  ${LDLIBS}

LOBJ = lattest.o latsym.o hkl_symmetry.o  \
 scala_util.o pointgroup.o pgscore.o \
 scsignificance.o lattice.o matvec_utils.o

lattest: ${LOBJ}
	${CXX} ${LFLAGS} -o lattest ${LOBJ}  ${LDLIBS}

JOBJ = testit.o 
testit: ${JOBJ}
	${CXX} ${LFLAGS} -o testit ${JOBJ}  ${LDLIBS}


ZOBJ = io_test.o InputAll.o CCP4base.o Preprocessor.o keywords.o Errors.o \
  range.o   matvec_utils.o scala_util.o Output.o
io_test: ${ZOBJ}
	${CXX}  ${LFLAGS} -o io_test ${ZOBJ}  ${LDLIBS}

MOBJ = xds2mtz.o xds_unmerge.o  Output.o jiffy.o Preprocessor.o Errors.o \
  hkl_controls.o controls.o hkl_symmetry.o hkl_unmerge.o hkl_datatypes.o \
  range.o matvec_utils.o scala_util.o rotation.o hash.o observationflags.o \
  icering.o keywords.o CCP4base.o pointgroup.o zone.o lattice.o latsym.o \
  getsubgroups.o probfunctions.o pgscore.o scsignificance.o writeunmergedmtz.o \
  mtz_utils.o cellgroup.o

xds2mtz: ${MOBJ}
	${CXX}  ${LFLAGS} -o xds2mtz ${MOBJ}  ${LDLIBS}

###test: ${WOBJ}
###	${CXX}  ${LFLAGS} -o test ${WOBJ}  ${LDLIBS}

PGOBJ = testpg.o pointgroup.o latsym.o scala_util.o matvec_utils.o lattice.o \
 Output.o hkl_symmetry.o getsubgroups.o  pgscore.o \
scsignificance.o

testpg: ${PGOBJ}
	${CXX} -o testpg ${PGOBJ}   ${LDLIBS}

HOBJ = testhkl.o hkl_unmerge.o  controls.o mtz_unmerge_io.o hash.o hkl_symmetry.o \
latsym.o scala_util.o  normalise.o range.o pointgroup.o \
 setscores.o spline.o pgscore.o scsignificance.o  lattice.o matvec_utils.o \
 average.o getsubgroups.o openinputfile.o \
interpretcommandline.o InputAll.o Preprocessor.o CCP4base.o Errors.o \
keywords.o Output.o hkl_controls.o hkl_merged_list.o \
 mtz_utils.o writeunmergedmtz.o intensitystatistics.o \
sysabszones.o zone.o hkl_datatypes.o  probfunctions.o score_datatypes.o icering.o \
jiffy.o rotation.o cellgroup.o observationflags.o \
 runthings.o makehkl_listscompatible.o printthings.o normaliseunmerge.o globalcontrols.o

testhkl: ${HOBJ}
	${CXX} $(LFLAGS) -o testhkl ${HOBJ}  ${LDLIBS}

RMOBJ = readmtz.o hkl_unmerge.o  controls.o mtz_unmerge_io.o hash.o hkl_symmetry.o \
latsym.o scala_util.o  normalise.o range.o pointgroup.o \
 setscores.o spline.o pgscore.o scsignificance.o  lattice.o matvec_utils.o \
 average.o getsubgroups.o openinputfile.o \
interpretcommandline.o InputAll.o Preprocessor.o CCP4base.o Errors.o \
keywords.o Output.o hkl_controls.o hkl_merged_list.o \
 mtz_utils.o writeunmergedmtz.o intensitystatistics.o \
sysabszones.o zone.o hkl_datatypes.o  probfunctions.o score_datatypes.o icering.o \
jiffy.o rotation.o cellgroup.o observationflags.o 
readmtz: ${RMOBJ}
	${CXX} $(LFLAGS) -o readmtz ${RMOBJ}  ${LDLIBS}


KOBJ = lt2.o latsym.o scala_util.o pointgroup.o  matvec_utils.o lattice.o
KEXE = lt2

lt2: ${KOBJ}
	${CXX} $(LFLAGS) -o ${KEXE} ${KOBJ}  ${LDLIBS}

ROBJ = reindex.o 
REXE = reindex

reindex: ${ROBJ}
	${CXX} $(LFLAGS) -o ${REXE} ${ROBJ}  ${LDLIBS}
 
TTOBJ = t.o \
  scala_util.o  hkl_datatypes.o  Output.o matvec_utils.o rotation.o \
  jiffy.o range.o string_util.o
TTEXE = t

t: ${TTOBJ}
	${CXX} $(LFLAGS) -o ${TTEXE} ${TTOBJ}  ${LDLIBS}

XOBJ = xia_output.o
XEXE = xia_output

${XEXE}: ${XOBJ}
	${CXX} $(LFLAGS) -o ${XEXE} ${XOBJ}  ${LDLIBS}

install: pointless
	cp ./pointless ${CBIN}/
