# Makefile for Pointless, Othercell & associate testing

# Compilers
CC = cc
CXX= c++

# Compiler flags
#OPT = -g
OPT = -O2

CXXFLAGS=  ${OPT}
CFLAGS= ${OPT}

# Link flags
LFLAGS= ${OPT}

# CCTBX locations
CCTBX=${CLIB}/cctbx
CCTBX_sources=${CCTBX}/cctbx_sources
#  Version 1 = 2006
#CCTBX_VERSION=2006
#  Version 2 = 2007
CCTBX_VERSION=2007

XCPPFLAGS=  -DCCTBX_VERSION=${CCTBX_VERSION}

# Clipper location
CLPLIB=${CLIB}
CLPR=${CLIB}/clipper

#<<<<< Includes
# CCP4
ICCP4=-I${CLIBS}

# Clipper
ICLPR=-I${CLPR} -I${CLPR}/clipper/core

# cctbx
ITBX=\
-I${CCTBX_sources}/boost \
-I${CCP4}/include \
-I${CCTBX_sources}/cctbx_project

INCLUDE=-I. ${ITBX} ${ICLPR} ${ICCP4}

#<<<<< Libraries
# CCP4
LCCP4=-L${CLIB} -lrfftw -lfftw -lccp4f -lccp4c -lmmdb -lm

# Clipper
LCLPR=-L${CLPLIB} -lclipper-ccp4 -lclipper-contrib -lclipper-minimol -lclipper-mmdb -lclipper-core 

# cctbx
LTBX= -L${CLIB} -lcctbx 

SLIB=

LDLIBS=${LTBX} ${LCLPR} ${LCCP4} ${SLIB}


CPPFLAGS=${XCPPFLAGS}  ${INCLUDE}
CCFLAGS = ${INCLUDE} 

TARGETS = pointless othercell
all: ${TARGETS}

####################################################################
#<<<<< Pointless
EXE = pointless
SRC=${EXE}.cpp

# Unique to Pointless
OBJ = \
	InputAll.o average.o centre.o checklatticecentering.o \
	checknullbatches.o choosesolution.o comparestring.o \
	completeness.o copymergedmtz.o datatypes.o expandfilename.o \
	filetype.o fix_merged_scores.o getCCsigfac.o globalcontrols.o \
	intensitystatistics.o io_files.o keywords.o listdir.o \
	makehkl_listscompatible.o normaliseunmerge.o \
	outputunmergeddata.o pgscore.o \
	printthings.o readallhklinfiles.o sca_unmerge.o \
	scsignificance.o setscores.o sysabsstats.o sysabstest.o \
	sysabszones.o writesummedlist.o writeunmergedmtz.o \
	xds_unmerge.o \
	testlauegroup.o testindex.o twintest.o

# Shared with Aimless
COBJ = \
	CCP4base.o Errors.o Output.o Preprocessor.o \
	cctbx_utils.o cellgroup.o checkcompatiblesymmetry.o \
	columnlabels.o controls.o eprob.o fileread.o \
	getsubgroups.o hash.o hkl_controls.o hkl_datatypes.o \
	hkl_merged_list.o hkl_symmetry.o hkl_unmerge.o icering.o \
	interpretcommandline.o jiffy.o latsym.o lattice.o \
	linearlsq.o matvec_utils.o \
	mtz_merge_io.o mtz_unmerge_io.o mtz_utils.o normalise.o \
	numbercomplete.o observationflags.o \
	openinputfile.o pointgroup.o probfunctions.o range.o \
	rotation.o runthings.o scala_util.o score_datatypes.o \
	spacegroupreindex.o spline.o string_util.o timer.o \
	zone.o tablegraph.o


OBJECTS = ${EXE}.o ${OBJ} ${COBJ}

# Unique to Pointless
HDR= \
	InputAll.hh average.hh centre.hh checklatticecentering.hh \
	checknullbatches.hh choosesolution.hh comparestring.hh \
	completeness.hh \
	copymergedmtz.hh datatypes.hh expandfilename.hh filetype.hh \
	fix_merged_scores.hh getCCsigfac.hh globalcontrols.hh \
	intensitystatistics.hh io_files.hh keywords.hh listdir.hh twintest.hh \
	makehkl_listscompatible.hh normaliseunmerge.hh outputunmergeddata.hh \
	pgscore.hh pointless.hh printthings.hh readallhklinfiles.hh \
	reindexscore.hh sca_unmerge.hh scale_model.hh scsignificance.hh \
	setscores.hh spline.hh sysabsstats.hh sysabstest.hh sysabszones.hh \
	version.hh writesummedlist.hh writeunmergedmtz.hh xds_unmerge.hh \
	testlauegroup.hh testindex.hh


# Shared with Aimless
CHDR = \
	CCP4base.hh Errors.hh InputBase.hh Output.hh Preprocessor.hh \
	cctbx_utils.hh cellgroup.hh checkcompatiblesymmetry.hh columnlabels.hh \
	controls.hh eprob.hh fileread.hh getsubgroups.hh hash.hh \
	hkl_controls.hh hkl_datatypes.hh hkl_merged_list.hh hkl_symmetry.hh \
	hkl_unmerge.hh icering.hh interpretcommandline.hh jiffy.hh latsym.hh \
	lattice.hh linearlsq.hh matvec_utils.hh mtz_merge_io.hh \
	mtz_unmerge_io.hh mtz_utils.hh normalise.hh numbercomplete.hh \
	observationflags.hh openinputfile.hh phaser_types.hh pointgroup.hh \
	probfunctions.hh range.hh rotation.hh runthings.hh scala.hh \
	scala_util.hh score_datatypes.hh spacegroupreindex.hh spline.hh \
	string_util.hh timer.hh util.hh zone.hh tablegraph.hh

# Unique to Pointless
SOURCE = \
	${EXE}.cpp \
	InputAll.cpp average.cpp centre.cpp checklatticecentering.cpp \
	checknullbatches.cpp choosesolution.cpp comparestring.cpp \
	completeness.cpp copymergedmtz.cpp datatypes.cpp expandfilename.cpp \
	filetype.cpp fix_merged_scores.cpp getCCsigfac.cpp globalcontrols.cpp \
	intensitystatistics.cpp io_files.cpp keywords.cpp listdir.cpp \
	twintest.cpp makehkl_listscompatible.cpp normaliseunmerge.cpp \
	outputunmergeddata.cpp pgscore.cpp \
	printthings.cpp readallhklinfiles.cpp saint.cpp sca_unmerge.cpp \
	scsignificance.cpp setscores.cpp sysabsstats.cpp sysabstest.cpp \
	sysabszones.cpp writesummedlist.cpp writeunmergedmtz.cpp \
	xds_unmerge.cpp \
	testlauegroup.cpp testindex.cpp


# common with Aimless
CSOURCE = \
	CCP4base.cpp Errors.cpp Output.cpp Preprocessor.cpp \
	cctbx_utils.cpp cellgroup.cpp checkcompatiblesymmetry.cpp \
	columnlabels.cpp controls.cpp eprob.cpp fileread.cpp \
	getsubgroups.cpp hash.cpp hkl_controls.cpp hkl_datatypes.cpp \
	hkl_merged_list.cpp hkl_symmetry.cpp hkl_unmerge.cpp icering.cpp \
	interpretcommandline.cpp jiffy.cpp latsym.cpp lattice.cpp \
	linearlsq.cpp matvec_utils.cpp \
	mtz_merge_io.cpp mtz_unmerge_io.cpp mtz_utils.cpp normalise.cpp \
	numbercomplete.cpp observationflags.cpp \
	openinputfile.cpp pointgroup.cpp probfunctions.cpp range.cpp \
	rotation.cpp runthings.cpp scala_util.cpp score_datatypes.cpp \
	spacegroupreindex.cpp spline.cpp string_util.cpp timer.cpp \
	zone.cpp tablegraph.cpp

DOC = pointless.html

${EXE}: ${OBJECTS} ${HDR}
	${CXX} ${LFLAGS} -o ${EXE} ${OBJECTS}  ${LDLIBS}

####################################################################
OC_OBJ = othercell.o latsym.o hkl_symmetry.o hkl_datatypes.o \
         scala_util.o pointgroup.o pgscore.o \
         scsignificance.o lattice.o matvec_utils.o getsubgroups.o \
	 Output.o zone.o probfunctions.o range.o\
         interpretcommandline.o  rotation.o cctbx_utils.o \
         Preprocessor.o CCP4base.o Errors.o jiffy.o  string_util.o \
	score_datatypes.o  spacegroupreindex.o

OC_SOURCE = othercell.cpp

othercell: ${OC_OBJ}
	${CXX} ${LFLAGS} -o othercell ${OC_OBJ}  ${LDLIBS}

####################################################################
# Export etc

export:
	/bin/rm -rf ../Export/
	mkdir ../Export
	cp ${SOURCE} ../Export/
	cp ${HDR} ../Export/
	cp ${CSOURCE} ../Export/
	cp ${CHDR} ../Export/
	cp ${OC_SOURCE} ../Export/
	cp ${DOC} ../Export/
	cp Makefile ../Export/Makefile.make
	mkdir ../Export/doc
	cp -R doc/html ../Export/doc

tar: export
	cd ../Export; tar cvf ../pointless.tar .
	cd ..; gzip -f pointless.tar;cp pointless.tar.gz ${scr0}

####################################################################

.SUFFIXES: .cpp .o

.cpp.o: ${HDR}
	${CXX} -c $(CXXFLAGS) $(CPPFLAGS) $<

.c.o:
	${CC} -c $(CFLAGS) $(CCFLAGS) $<

.f.o:
	${FCOMP}  -c $<


clean: 
	rm *.o ${TARGETS}  ${POINTLESS_LIB} *.gch

####################################################################
