// Testing Saint input

#include <vector>


#include <string>
#include <iostream>
#include <fstream>

// Clipper
#include <clipper/clipper.h>
using clipper::Vec3;
using clipper::Mat33;
using clipper::Message;
using clipper::Message_fatal;

#include "rotation.hh"
#include "hkl_datatypes.hh"
#include "matvec_utils.hh"
#include "range.hh"
#include "score_datatypes.hh"
#include "hash.hh"


using namespace scala;

//--------------------------------------------------------------
double Vlength(const DVect3& v)
{return sqrt(v*v);}
//--------------------------------------------------------------
std::string Line(const std::string buf)
// Extract line from buffer, removing any trailing Cr or Lf characters
{
  std::string line(buf);
  size_t ll = line.size();
  while (ll > 0) {
    if (line[ll-1] != '\n' && line[ll-1] != '\r') {break;}
    ll--;
  }
  return line.substr(0,ll);
}
//--------------------------------------------------------------
class MGvector {

public:
  MGvector(){}
  MGvector(const std::string& filename, const std::string& name,
	   const std::string& colour)
  {
    outstream.open(filename.c_str());

    outstream << "\n\ndata_vectors_"+name+"\n";

    outstream << 
      std::string("\nloop_\n")+
      "_display_objects.line_style\n"+
      "_display_objects.line_colour\n"+
      "_display_objects.line_width\n"+
      "_display_objects.label_mode\n"+
      "_display_objects.label_colour\n"+
      "\"solid\" \""+colour+"\" 1 \"label centre\" \"yellow\"\n\n";

    outstream << 
      std::string("\nloop_\n")+
      "_vector.tag\n"+
      "_vector.start_x\n"+
      "_vector.start_y\n"+
      "_vector.start_z\n"+
      "_vector.end_x\n"+
      "_vector.end_y\n"+
      "_vector.end_z\n";
  }

  ~MGvector() {
    if (outstream.is_open()) {
      outstream.close();
    }
  }

  void Vectors(const std::vector<Vec3<double> >& v, const Vec3<double>& v0) {
    for (size_t i=0;i<v.size();++i) {
      clipper::String tag;
      tag = "";
      outstream << "\""+tag+"\" "
		<< v0[0] << " "<< v0[1] << " "<< v0[2] << "  "
		<< v[i][0] << " "<< v[i][1] << " "<< v[i][2] << "\n";
    }
    outstream << "\n";
  }

  void Vectors2(const std::vector<Vec3<double> >& v1, const std::vector<Vec3<double> >& v2,
		const Vec3<double>& v0) {
    ASSERT (v1.size() == v2.size());
    for (size_t i=0;i<v1.size();++i) {
      clipper::String tag(i);
      tag = "";
      outstream << "\""+tag+"\" "
	<< v1[i][0] << " "<< v1[i][1] << " "<< v1[i][2]<< "  "
	<< v0[0] << " "<< v0[1] << " "<< v0[2] 
	<< "\n";
      outstream << "\""+tag+"\" "
	<< v0[0] << " "<< v0[1] << " "<< v0[2] << "  "
	<< v2[i][0] << " "<< v2[i][1] << " "<< v2[i][2]
	<< "\n";
    }
    outstream << "\n";
  }

private:
  std::ofstream outstream;
};
//--------------------------------------------------------------
class Brukerp4p {
  // Bruker p4p file info
public:
  Brukerp4p(){}
  Brukerp4p(const std::string& filename);

  DMat33 UB() const {return UBmat;}
  Scell  Cell() const {return Scell(celld);}
  double Z0_2theta()const {return z0_2theta;}
  double Z0_omega()const {return z0_omega;}
  double Z0_chi()const {return z0_chi;}

private:
  DMat33 UBmat; // [UB]
  double z0_2theta, z0_omega, z0_chi;  // zero offsets
  std::vector<double> celld;
};
//--------------------------------------------------------------
Brukerp4p::Brukerp4p(const std::string& filename)
{
  std::ifstream instream;
  instream.open(filename.c_str());
  if (!instream) {
    Message::message(Message_fatal
		     ("P4P: cannot open file "+filename));
  }
  celld.resize(6);

  clipper::String line;
  while (true) {
    std::getline(instream, line );
    if (instream.eof()) break;  // end of file
    line = Line(line);  // remove any trailing lf or cr characters
    
    if (line.substr(0,3) == "ORT") {
      int i;
      if (line[3] == '1') {
	i = 0;
      } else if (line[3] == '2') {
	i = 1;
      } else if (line[3] == '3') {
	i = 2;
      }
      for (int j=0;j<3;++j) {
	UBmat(i,j) = std::atof(line.substr(7+j*16,16).c_str());
      }
    } else  if (line.substr(0,5) == "ZEROS") {
      z0_2theta = std::atof(line.substr(6,11).c_str());
      z0_omega  = std::atof(line.substr(17,11).c_str());
      z0_chi    = std::atof(line.substr(28,11).c_str());
    } else  if (line.substr(0,6) == "CELL  ") {
      
      celld[0] = std::atof(line.substr(6,10).c_str());
      celld[1] = std::atof(line.substr(16,10).c_str());
      celld[2] = std::atof(line.substr(26,10).c_str());
      celld[3] = std::atof(line.substr(36,10).c_str());
      celld[4] = std::atof(line.substr(46,10).c_str());
      celld[5] = std::atof(line.substr(56,10).c_str());
    }
  }
}
//--------------------------------------------------------------
void DecomposeUB(const DMat33& UB, DMat33& U, Scell& cell)
{
  std::cout << "\nDecomposeUB:\n" << UB.format()
	    << "\nDet[UB] = " << UB.det() << "\n\n";
  DMat33 MtM = UB.transpose()*UB;
  std::cout << "\n[UB]T[UB] metric tensor = [B]T[B]\n" << MtM.format()  << "\n";

  MetricTensor MT(MtM);
  cell = Scell(MT, false);
  std::cout << "\n[B] from metric tensor\n"
	    << cell.Bmat().format() << "\n";
  std::cout <<"Equivalent cell: " << cell.format() << "\n\n";

  U = UB * cell.Bmat().inverse();
  std::cout <<"[U] orientation matrix\n" << U.format()
	    <<"\nDet[U] = " << U.det() 
	    << "\n\n";

}
//--------------------------------------------------------------
class SaintRun {
  //  A "run" or "sweep" from a Saint file, ie information about a set of reflections
  //  with the same scan axis and non-scan goniostat axes
  //
  // The SAINT file contains information which can be used to extract the unit cell,
  // & crystal orientation information. The only external information needed in the
  // wavelength which cannot be inferred from the file.
  //
  // Each reflection record contains the following information on cell & orientation:
  //   h,k,l        indices
  //   s0r(3)       direction cosines of incident beam s0, projected on reciprocal
  //                lattice axes. This in the reciprocal axis frame "r", not
  //                necessarily orthogonal
  //   s2r(3)       direction cosines of secondary beam in reciprocal axis frame
  //   rot          rotation angle (omega or phi) degrees
  //   iaxis        scan axis 2=omega, 3=phi
  //   istl         sin theta/lambda * 10000
  //   chi          chi angle degrees
  //   other        phi or omega, non-scan angle degrees
  //
  //  s = [R] [U] [F^-1] [F] [K] [C] h   = [R] [U] [B] h 
  //  l      z   x      r         frames
  //
  //  orthogonalisation matrix [B] = [K][C] where [C] = diag(a*,b*,c*)
  //  [F] = [K]T 
  //  ie [F] takes a unit orthogonal vector v -> f (cosines on a*, b*, c*)
  //     f = (v.a*/|a*|  v.b*/|b*|  v.c*/|c*|)
  //       =  [F] v
  //
  //     [F] = (a*/|a*|) = [K]T = [B][C^-1]
  //           (b*/|b*|)
  //           (c*/|c*|)
  //  
  //  Coordinate frames:
  //    r   reciprocal axis frame
  //    x   crystal frame
  //    z   zero angle frame (all goniostat angles zero)
  //    l   laboratory frame
  //
  // Determination of parameters
  //  1) the cell is determined from sin theta/ lambda values
  //     This may be overridden by input values
  //  2) the orientation matrix [U] is determined from the incident beam cosines
  //  3) the wavelength is determined from {U][B]: this may be overridden from input
  //  
public:
  SaintRun(){}

  SaintRun(const int& Ibatch, const int& Iaxis, const double& Chi, const double& Other,
	    const int& Icryst, const double& Swing);

  void init(const int& Ibatch, const int& Iaxis, const double& Chi, const double& Other,
	    const int& Icryst, const double& Swing);

  bool SameRun(const int& Ibatch, const int& Iaxis,
	       const double& chi, const double& other,
	       const int& icryst, const double& swing);


  void StoreReflection(const IVect3& hkl, const int& batch,
		       const DVect3& s0r, const DVect3& s2r,
		       const double& rot, const int& istl);

  scala::Scell CalcCell();
  // calculate cell (returned) from dstar information, linear LSQ

  void CalcResults();

  void SetBatchOffset(const int& batchOffset) {batchoffset = batchOffset;}
  int BatchOffset() const {return batchoffset;}
  int MaxBatch() const;  // maximum batch number including offset
  int Nbatches() const { return nbatches;}

  std::string ScanAxis() const;
  double Chi() const {return chi;}
  double Other() const {return other;}
  double Swing() const {return swing;}

  DMat33 Umatrix() const {return U;}

  double Wavelength() const {return wavelength;}
  double SdWavelength() const {return sdwavelength;}
  scala::Scell Cell() const {return cell;}

  void SetWavelength(const double& wvl) {wavelength = wvl; sdwavelength=0.0;}
  void SetCell(const scala::Scell& Cell) {cell = Cell;}

  static DMat33 Rmatrix(const double& omegar, const double& chir, const double& phir);
  static DVect3 s0v() {return s0;}

  std::vector<CMtz::MTZBAT> MakeBatch(const float& wavelength);

private:
  double BasicAngleD(const double& aa);
  bool SameAngle(const double& a1, const double& a2);

  struct CompareMtzBat {
    // This construct seems to be a way of getting pointer-to-function
    // into the argument for sort. Copied from the Web.
    bool operator()(const CMtz::MTZBAT& a, const CMtz::MTZBAT& b);
  };


  int ibatch;  // batch number from file
  int iaxis;   //  scan axis 2=omega, 3=phi
  double chi;  // chi angle, degrees
  double other; // non-scan axis (phi or omega), degrees
  double icryst;
  double swing;
  std::vector<scala::Range> rotRange;

  double omegar, chir, phir; // radians
  //  Lists of all reflections from file
  std::vector<DVect3> s0rv;   // unit s0 in reciprocal axis (r) frame from file
  std::vector<DVect3> s2rv;   // unit s2 in reciprocal axis frame from file
  std::vector<IVect3> hklv;    // hkl
  std::vector<int> istlv;      // sin theta/lambda * 10000
  std::vector<DMat33> Rv;     // Goniostat rotation matrix [Omega][Chi][Phi]


  // results
  bool calculated;  // true if result matrices etc have been calculated
  DMat33 U;
  scala::Scell cell;
  double wavelength;
  double sdwavelength;

  // Batch information
  int nbatches;
  hash_table lookup;
  int table_size;
  int batchoffset;

  //  Saint goniostat things: axis directions
  // Note: "Cambridge" axis system
  //   z along principle rotation axis (e1 = Omege in ths case)
  //   x in plane containing z & s0
  //   y along s0 x e1
  //   source vector s0 anti-parallel to beam ie approximately along -x
  static DVect3 e1;  // Omega along z
  static DVect3 e2;  // Chi along x
  static DVect3 e3;  // Phi along z
  static DVect3 s0;  // s0 along x

  static int nxpix, nypix;
};  // SaintRun

//--------------------------------------------------------------
bool  SaintRun::CompareMtzBat::operator()(const CMtz::MTZBAT& a, const CMtz::MTZBAT& b)
{return (a.num < b.num);}
//--------------------------------------------------------------
DVect3 SaintRun::e1(0., 0., 1.0);  // Omega along +z
DVect3  SaintRun::e2(-1.0, 0.0, 0.);  // Chi along -x
DVect3  SaintRun::e3(0., 0., -1.0);  // Phi along -z
DVect3  SaintRun::s0(-1.0,0.0,0.0);  // s0 along x
int SaintRun::nxpix = 512;
int SaintRun::nypix = 512;
//--------------------------------------------------------------
SaintRun::SaintRun(const int& Ibatch, const int& Iaxis,
		   const double& Chi, const double& Other,
		   const int& Icryst, const double& Swing)
{
  init(Ibatch, Iaxis, Chi, Other, Icryst, Swing);
}
//--------------------------------------------------------------
  void SaintRun::init(const int& Ibatch, const int& Iaxis, const double& Chi, const double& Other,
		    const int& Icryst, const double& Swing)
{
  ibatch = Ibatch;
  iaxis = Iaxis;
  chi = BasicAngleD(Chi);
  other = BasicAngleD(Other);
  icryst = Icryst;
  swing = BasicAngleD(Swing);

  double phi = 0.0;
  double omega = 0.0;
  if (iaxis == 2) {
    // Omega scan
    phi = other;
  } else {
    // Phi scan
    omega = other;
  }
  omegar = clipper::Util::d2rad(omega);
  chir = clipper::Util::d2rad(chi);
  phir = clipper::Util::d2rad(phi);

  const int TABLE_SIZE = 5000;
  lookup.set_size(TABLE_SIZE);
  table_size = lookup.Size();  // smallest prime >= set size
  nbatches = 0;
  batchoffset = 0;
}
//--------------------------------------------------------------
double SaintRun::BasicAngleD(const double& aa)
// Reduce angle in degrees to range -180 - +180
{
  double a = aa;
  while (a <=  -180.) {a+=360.;}
  while (a >  +180.) {a-=360.;}
  return a;
}
//--------------------------------------------------------------
bool SaintRun::SameAngle(const double& a1, const double& a2)
// true is angles a1 & a2 (degrees) are the same within tolerance (modulo 360)
//  a2 is assumed to be already "Basic" ie in range 0-360
{
  const double TOL = 0.01;
  // True if angles are the same modulo 360
  return (std::abs(BasicAngleD(a1) -a2) < TOL);  
}
//--------------------------------------------------------------
bool SaintRun::SameRun(const int& Ibatch, const int& Iaxis,
		       const double& Chi, const double& Other,
		       const int& Icryst, const double& Swing)

{
  if (Ibatch != ibatch) return false;
  if (Icryst != icryst) return false;
  if (Iaxis != iaxis) return false;
  if (!SameAngle(Chi, chi)) return false;
  if (!SameAngle(Other, other)) return false;
  if (!SameAngle(Swing, swing)) return false;
  return true;
}
//--------------------------------------------------------------
  void SaintRun::StoreReflection(const IVect3& hkl, const int& batch,
			       const DVect3& s0r, const DVect3& s2r,
			       const double& rot, const int& istl)
{
  hklv.push_back(hkl);
  s0rv.push_back(s0r);
  s2rv.push_back(s2r);
  istlv.push_back(istl);
  if (iaxis == 2) {
    // Omega scan
    omegar = clipper::Util::d2rad(rot);
  } else {
    // Phi scan
    phir = clipper::Util::d2rad(rot);
  }
  // Goniostat rotation matrix
  Rv.push_back(Rmatrix(omegar,chir,phir));

  // do we have this batch already?
  int ib = lookup.lookup(batch);
  if (ib < 0) {
    // New batch, add to list
    nbatches++;
    if (nbatches > 0.8*table_size) {
      Message::message(Message_fatal
		       ("SCAIO: increase TABLE_SIZE"));
    }
    lookup.add(batch, nbatches-1);
    rotRange.push_back(scala::Range());
    ASSERT (int(rotRange.size()) == nbatches);
    ib = nbatches-1;
  }
  rotRange.at(ib).update(rot);
  //  std::cout << "Batch, rot " << batch << " " << rot << "\n";
}
//--------------------------------------------------------------
DMat33 SaintRun::Rmatrix(const double& omegar, const double& chir, const double& phir)
{
  // Total goniostat rotation
  clipper::Rotation R(scala::Euler_explicit
		      (e1, omegar,
		       e2, chir,
		       e3, phir));
  return R.matrix();
}
//--------------------------------------------------------------
int SaintRun::MaxBatch() const
// maximum batch number including offset
{
  int maxbatch = -1;
  for (int i=0;i<nbatches;++i) {
    maxbatch = Max(maxbatch, lookup.number(i));
  }
  return maxbatch;
}
//--------------------------------------------------------------
std::string SaintRun::ScanAxis() const
{
  return (iaxis==2) ? "OMEGA" : "PHI";
}
//--------------------------------------------------------------
scala::Scell SaintRun::CalcCell()
// calculate cell (returned) from dstar information, linear LSQ
//
// For each reflection, we have an observational equation
//   e = hT [M] h - q
// where q = |d*|^2  = 4 (sin theta/lambda)^2 (from the file)
//       h = (h k l)
//       [M] is the reciprocal metrix tensor = [B]T [B]
//
// We can solve for [M] as a 6-vector m,
//    elements of m indexed as m(l) = m(ij) for l = 0,5 or j=0,2; i=0,j
//    ie l = j(j+1)/2 + i
// Then  m = [H]^-1 r
// where [H]kl = Sum(n=1,N)  h(ik)n h(jk)n h(il)n h(jl)n
//          ik,jk are i,j corresponding to k
//          il,jl are i,j corresponding to l
//       r(ij) = Sum(n=1,N)   h(ik)n h(jk)n qn = r(l)
{
  clipper::Matrix<double> H(6,6,0.0);
  std::vector<double> r(6,0.0);

  for (int n=0;n<hklv.size(),++n){
    IVect3 hkl = hklv[n];
    for (int j=0;j<3;++j){
      for (int i=0;i<3;++i){ // loop for vector elements & matrix column
	int l = (j*(j+1))/2 + i;
	int hij = (hkl[i]*hkl[j])
	  r[l] += (double(istlv[i])/5000.) * double(hij);
	for (int jj=0;jj<3;++jj){
	  for (int ii=0;ii<3;++ii){ // loop for matrix row
	    int k = (jj*(jj+1))/2 + ii;
	    H(k,l) += double(hij*hkl[ii]*hkl[jj]);
	  }
	}
      }
    }
  }
  std::vector<double> m = H.solve(r);  // solve for m
  DMat33 M; // reciprocal metric tensor
  for (int j=0;j<3;++j){
    for (int i=0;i<3;++i){ 
      int l = (j*(j+1))/2 + i;
      M(i,j) = m(l);
      if (i != j) M(j,i) = m(l);
    }}
  cell = scala::Scell(M,false);
  return cell;
}
//--------------------------------------------------------------
void SaintRun::CalcResults()
//  
{
  ASSERT ((s0rv.size() == Rv.size()) &&
	  (s2rv.size() == hklv.size()) &&
	  (s2rv.size() == istlv.size()) &&
	  (s0rv.size() == hklv.size()));

  // Cell from dstar
  CalcCell();
  //^
  std::cout << "Cell from sin theta/lambda: "<<cell.format()<<"\n";

  DMat33 F =  Fcell.Bmat().transpose();


  // unit s0 in r frame from back-rotated [R]^-1 s0
  std::vector<DVect3> s0xv(s0rv.size());
  for (int i=0;i<s0rv.size();++i){
    s0xv[i] = Rv[i].inverse() * s0;  // s0x = [R]^-1 s0
  }
  // Orthogonalised incident beam vector s0xr = [F]^-1 s0r 
  std::vector<DVect3> s0xrv(s0rv.size());a
  for (int i=0;i<s0rv.size();++i){
    s0xv[i] = Rv[i].inverse() * s0;  // s0r = [R]^-1 s0
  }




  double E;
  // [U][F^-1] from superposition
  DMat33 UFi = MVutil::LsqTransform(s0rv, s0xv, E, true);
  // [F^-1]T [F^-1]
  DMat33 FiTFi = UFi.transpose()*UFi;
  // Skew part of cell from real metric tensor
  scala::Scell Fcell(FiTFi, true);

  // [F]
  DMat33 F =  Fcell.Bmat().transpose();
  U = UFi * F;
  
  std::vector<DVect3> sxv(s2rv.size());
  DVect3 sx;
  DMat33 Fi = F.inverse();  // [F]^-1
  for (int i=0;i<s2rv.size();++i) {
    sx = Fi*(s0rv[i]+s2rv[i]);
    sxv[i] =  (double(istlv[i])/5000.)*(sx.unit());
  }
  DMat33 B = MVutil::LsqTransform(hklv, sxv, E);
  // 
  cell = scala::Scell(B.transpose()*B, false);
  // [UB]
  DMat33 UB = U * B;

  // Calculate wavelength
  // When the diffraction vector s is in the diffracting position, it makes an angle
  // of (90 - theta) with the direct beam
  scala::MeanSD mnL;
  for (int i=0;i<hklv.size();++i) {
    DVect3 s = Rv[i] * UB * hklv[i];
    double sintheta = s * s0 /sqrt(s*s);
    double lambda = 2.*sintheta/dstarv[i];
    mnL.Add(lambda);
  }
  wavelength = mnL.Mean();
  sdwavelength = mnL.SD();
}
//--------------------------------------------------------------
  std::vector<CMtz::MTZBAT> SaintRun::MakeBatch(const float& wavelength)
  // Make batch data
{
  //  jumpax is the index (1,2,3) of the reciprocal axis closest to the
  //  rotation axis. We can get this by back-rotating the rotation axis e1
  //  into crystal space with the inverse orientation matrix [U]^-1.
  //  Then jumpax corresponds to the largest component
  DVect3 UinvE;
  if (iaxis == 2) {
    // Omega scan
    UinvE = U.inverse() * e1;
  } else {
    // Phi scan
    UinvE = U.inverse() * e3;
  }
  int jumpax=0;
  double temp = 0.0;
  for (int i=0;i<3;i++) {
    if (std::abs(UinvE[i]) > temp)  {
      jumpax = i+1;
      temp = std::abs(UinvE[i]);
    }
  }
  CMtz::MTZBAT BHeader;   // header structure
  //+  BHeader.num = BatchNumber;        /**< batch number */
  strcpy(BHeader.title, "SAINT data");   /**< batch title */	      
  strcpy(BHeader.gonlab[0], "OMEGA   ");    /**< names of the three axes */
  strcpy(BHeader.gonlab[1], "CHI     ");    // 8-characters only! 
  strcpy(BHeader.gonlab[2], "PHI     ");
  BHeader.iortyp = 0;                  /**< type of orientation block (for 
					    possible future use, now = 0) */
  /**< refinement flags for cell (constraint flags)  no constraints*/
  for (int i=0;i<6;i++) {BHeader.lbcell[i] = -1;}
  BHeader.misflg = 0;                  /**< number of phixyz used (0, 1, or 2) */
  BHeader.jumpax = jumpax;             /**< reciprocal axis closest to rotation
					  axis */
  BHeader.ncryst=icryst;               /**< crystal number */
  BHeader.lcrflg = 0;                  /**< mosaicity model: 0 = isotropic, 
					  1 = anisotropic */
  BHeader.ldtype = 2;                  /**< type of data: 2D (1), 3D (2), or 
					  Laue (3) */
  if (iaxis == 2) {
    // Omega scan
    BHeader.jsaxs = 1;                 /**< goniostat scan axis number */
    for (int i=0;i<3;i++) {BHeader.scanax[i] = e1[i];}     /**< rotation axis in lab frame */
  } else {
    BHeader.jsaxs = 3;                 /**< goniostat scan axis number */
    for (int i=0;i<3;i++) {BHeader.scanax[i] = e3[i];}     /**< rotation axis in lab frame */
  }
  BHeader.nbscal = 0;                  /**< number of batch scales & Bfactors
					  (0 if unset) */
  BHeader.ngonax = 3;                  /**< number of goniostat axes */
  BHeader.lbmflg = 0;                  /**< flag for type of beam info:
					  = 0 for alambd, delamb
					  = 1 also delcor, divhd, divvd */
  BHeader.ndet = 1;                    /**< number of detectors (current maximum
					  2) */
  BHeader.nbsetid = 1;                 /**< dataset id - should be pointer? */
  for (int i=0;i<6;i++) {BHeader.cell[i] = cell[i];}   /**< cell dimensions */
  /**< orientation matrix U in Fortranic order, i.e. U(1,1), U(2,1) ... */
  int k=0;
    for (int i=0;i<3;i++) {
      for (int j=0;j<3;j++) {
	BHeader.umat[k++] = U(j,i);
      }}
    /**< missetting angles at beginning and end of oscillation */
    for (int i=0;i<2;i++) {
      for (int j=0;j<3;j++) {
	BHeader.phixyz[i][j] = 0.0;
      }}
    BHeader.crydat[0] = 0.0;              /**< mosaicity */
    for (int i=1;i<12;i++) {BHeader.crydat[i] = 0.0;}
    /**< datum values of goniostat axes */
    for (int i=0;i<3;i++) {BHeader.datum[i] = 0.0;}
    //+    BHeader.phistt = rotRange.min();         /**< start of phi relative to datum */
    //+    BHeader.phiend = rotRange.max();         /**< end of phi relative to datum */
    BHeader.time1 = 0.0;                /**< start time */
    BHeader.time2 = 0.0;                /**< stop time */
    BHeader.bscale = 0.0;               /**< batch scale */
    BHeader.bbfac = 0.0;                /**< batch temperature factor */
    BHeader.sdbscale = 0.0;             /**< sd bscale */
    BHeader.sdbfac = 0.0;               /**< sd bbfac */
    //+    BHeader.phirange = osc_range;       /**< phi range */
    /**< vector 1 ("Cambridge" laboratory axes defining ngonax goniostat axes */
    for (int i=0;i<3;i++) {BHeader.e1[i] = e1[i];}  /**< vector 1 */
    for (int i=0;i<3;i++) {BHeader.e2[i] = e2[i];}    /**< vector 2 */
    for (int i=0;i<3;i++) {BHeader.e3[i] = e3[i];}    /**< vector 3 */
    for (int i=0;i<3;i++) {BHeader.source[i] = s0[i];} /**< idealised source vector */
    for (int i=0;i<3;i++) {BHeader.so[i] = s0[i];}     /**< source vector (unit length)*/
    BHeader.alambd = wavelength;        /**< wavelength (A) */
    BHeader.delamb = 0.0;               /**< dispersion (deltalambda / lambda) */
    BHeader.delcor = 0.0;               /**< correlated component */
    BHeader.divhd = 0.0;                /**< horizontal beam divergence */
    BHeader.divvd = 0.0;                /**< vertical beam divergence */
    BHeader.dx[0] = 0.0;                /**< xtal to detector distance */
    BHeader.dx[1] = 0.0;                /**< xtal to detector distance */
    BHeader.theta[0] = swing;           /**< detector tilt angle */
    BHeader.theta[1] = 0.0;             /**< detector tilt angle */
    /**< min & max values of detector coords (pixels) */
    BHeader.detlm[0][0][0] = 1.;
    BHeader.detlm[0][0][1] = nxpix;
    BHeader.detlm[0][1][0] = 1.;
    BHeader.detlm[0][1][1] = nypix;
    BHeader.detlm[1][0][0] = 0.;
    BHeader.detlm[1][0][1] = 0.;
    BHeader.detlm[1][1][0] = 0.;
    BHeader.detlm[1][1][1] = 0.;
    // End of MTZBAT

    std::vector<CMtz::MTZBAT> batchHeaders;
    for (int i=0;i<nbatches;++i) {
      BHeader.num = lookup.number(i);        /**< batch number */
      ASSERT (BHeader.num > 0);
      BHeader.phistt = rotRange[i].min();         /**< start of phi relative to datum */
      BHeader.phiend = rotRange[i].max();         /**< end of phi relative to datum */
      float osc_range = rotRange[i].max() - rotRange[i].min();
      BHeader.phirange = osc_range;       /**< phi range */  
      batchHeaders.push_back(BHeader);
    }
    ASSERT (batchHeaders.size() == nbatches);

    // sort on batch number
    std::sort(batchHeaders.begin(), batchHeaders.end(), CompareMtzBat());
    // Force end rot value of each batch to match start rot of next
    // ie batches should be contiguous
    for (int i=0;i<nbatches-1;++i) {
      float rotLast = batchHeaders[i].phiend;
      float rotNext = batchHeaders[i+1].phistt;
      float avrot = 0.5*(rotLast+rotNext);
      batchHeaders[i].phiend = avrot;
      batchHeaders[i+1].phistt = avrot;
      batchHeaders[i].phirange = batchHeaders[i].phiend - batchHeaders[i].phistt;
    }
    batchHeaders[nbatches-1].phirange =
      batchHeaders[nbatches-1].phiend - batchHeaders[nbatches-1].phistt;

    return batchHeaders;
  }
//--------------------------------------------------------------
//--------------------------------------------------------------
int main() {

  std::string scaname;
  std::cin >> scaname;
  double a,b,c,al,be,ga;
  std::cin >> a>>b>>c>>al>>be>>ga; //cell
  double wvl = 1.54188; // Cu Ka
  std::string p4pname;
  std::cin >> p4pname;
  double rot1, rot2;
  //  std::cin >> rot1 >> rot2;  // rot limits
  
  // Interpret p4p file to get orientation matrix [UB] and zeroes
  DMat33 bU, bK, bUB;
  if (p4pname != "null") {
    std::cout <<"Info from p4p file ----------\n\n";
    Brukerp4p P4P(p4pname);
    bUB = P4P.UB();
    std::cout << "[UB] from p4p file\n" << bUB.format() << "\n\n";
    Scell bcell;
    DecomposeUB(bUB,bU, bcell);
    std::vector<double>  breccell = bcell.ReciprocalCell();
    // [C] = diag(a*,b*,c*)
    Mat33<double> bC = MVutil::SetDiagCMat33(DVect3(breccell[0],breccell[1],breccell[2]));
    // [K] = [B] [C]^-1
    bK = bcell.Bmat() * bC.inverse();
    std::cout << "[K] from p4p cell\n" << bK.format()
	      << "\nDet[K] = " << bK.det() << "\n";
    std::cout <<"Cell from file: " << P4P.Cell().format() << "\n";
    std::cout <<"\nEnd info from p4p file ----------\n\n";
  }

  Scell cell(a,b,c,al,be,ga);
  std::cout << "\n**** Input cell: " << cell.format() << "\n";
  std::cout << "\n[B] from input cell\n"
	    << cell.Bmat().format() << "\n"
	    << "Det = " << cell.Bmat().det() << "\n\n";


  // Diffraction equation
  //   d = [Omega][Chi][Phi] [U] [B] h
  //          where d is diffraction vector on Ewald sphere
  //     = [R] [U] [B] h 
  //     = [R] [U] [K] [C] h   where [B] = [K] [C]  and [C] = Diag(a*,b*,c*)
  //
  //   [U] is orientation matrix (orthonormal)
  //   [K] is cell skew matrix
  // Cell matrices etc
  // [B] = [K] [C]
  Mat33<double> Bmat = cell.Bmat();
  std::vector<double>  reccell = cell.ReciprocalCell();
  // [C] = diag(a*,b*,c*)
  Mat33<double> Cmat = MVutil::SetDiagCMat33(DVect3(reccell[0],reccell[1],reccell[2]));
  // [K] = [B] [C]^-1
  Mat33<double> Kmat = Bmat * Cmat.inverse();
  //^
  std::cout << "[K] from input cell\n" << Kmat.format()
	    << "\nDet[K] = " << Kmat.det() << "\n";

  // [L] = lamda [C]  ie in dimensionless r.l.u
  DMat33 Lmat = MVutil::SetDiagCMat33(DVect3(wvl,wvl,wvl)) * Cmat;


  std::ifstream scain;
  scain.open(scaname.c_str());
  if (!scain) {
    Message::message(Message_fatal
		     ("SCAIO: cannot open file "+scaname));
  }


  Vec3<double> s0r;   // unit s0 in reciprocal axis frame from file
  Vec3<double> s2r;   // unit s2 in reciprocal axis frame

  Mat33<double> Rmat;    // Goniostat rotation matrix
  Mat33<double> RotInv;  //  ... and its inverse

  // Two corresponding vector lists:
  //   s0rv  s0r s0 in reciprocal axis frame from file
  //   s0xv  s0x s0 back-rotated by setting angles [R]^-1 s0
  //  s0x = [U][K] s0r
  std::vector<DVect3> s0rv;
  std::vector<DVect3> s0xv;

  // Two corresponding vector lists:
  //   s0rxf  s0 in reciprocal axis frame from file
  //   s0brt  s0 back-rotated by setting angles [R]^-1 s0 
  std::vector<clipper::Coord_orth> s0rxf;
  std::vector<clipper::Coord_orth> s0brt;


  std::vector<DVect3> s2rv;
  std::vector<DVect3> srv;
  std::vector<DVect3> hrv;

  std::vector<DMat33> Rv; // [R] for each reflection
  std::vector<double> rotv;

  // Add in origin vector to both lists (a few times!)
  for (int i=0;i<30;i++)  {
    s0rxf.push_back(clipper::Coord_orth(0.,0.,0.));
    s0brt.push_back(clipper::Coord_orth(0.,0.,0.));
  }

  int i = 0;
  double chiL, otherL, swingL;
  int icrystL;
  bool first = true;

  int iaxis;
  int istl;
  int pksum;
  int bgsum;
  int icryst;

  double xd;
  double yd;
  double zfr;
  double xdp;
  double ydp;
  double zfp;
  double LPap;
  double corr;
  double accExp;
  double swing;
  double rot;
  double rotrel;
  double corL;
  double xg;
  double yg;
  double other;

  double omega, chi, phi; // degrees
  double omegar, chir, phir; // radians

  Range rotRange;
  Range rotrelRange;

  SaintRun SR;
  SR.SetCell(cell);

  clipper::String line;
  while (true) {
    std::getline(scain, line );
    if (scain.eof()) break;  // end of file
    line = Line(line);  // remove any trailing lf or cr characters
    // Indices
    int h = std::atoi(line.substr(0, 4).c_str());
    int k = std::atoi(line.substr(4, 4).c_str());
    int l = std::atoi(line.substr(8, 4).c_str());

    s0r[0] = std::atof(line.substr(32,8).c_str()); // incident beam
    s2r[0] = std::atof(line.substr(40,8).c_str()); // secondary beam
    s0r[1] = std::atof(line.substr(48,8).c_str()); // y
    s2r[1] = std::atof(line.substr(56,8).c_str());
    s0r[2] = std::atof(line.substr(64,8).c_str()); // z
    s2r[2] = std::atof(line.substr(72,8).c_str());

    xd = std::atof(line.substr(83,7).c_str()); // detector pixels
    yd = std::atof(line.substr(90,7).c_str());
    zfr = std::atof(line.substr(97,8).c_str()); // Observed frame number
    
    xdp = std::atof(line.substr(105,7).c_str()); // Predicted x,y
    ydp = std::atof(line.substr(112,7).c_str()); //   detector pixels
    zfp = std::atof(line.substr(119,8).c_str()); // Predicted frame
    
    LPap = std::atof(line.substr(127,6).c_str()); // Applied LP etc
    corr = std::atof(line.substr(133,5).c_str()); // profile correlation
    
    accExp = std::atof(line.substr(138,7).c_str()); // accumlated hours exposure
    swing = std::atof(line.substr(145,7).c_str());  // detector swing degrees
    rot = std::atof(line.substr(152,7).c_str()); // scan axis value (omega or phi) degrees
    iaxis = std::atoi(line.substr(159,2).c_str());  // scan axis 2=omega, 3=phi
    istl = std::atoi(line.substr(161,5).c_str());  // sin theta/lambda * 10000
    pksum = std::atoi(line.substr(166,9).c_str());  // peak sum
    bgsum = std::atoi(line.substr(175,7).c_str());  // background sum
    rotrel = std::atof(line.substr(182,7).c_str()); // relative rotation degrees
    icryst = std::atoi(line.substr(189,4).c_str());  // crystal number
    corL = std::atof(line.substr(213,6).c_str());  // Lorentz factor
    xg = std::atof(line.substr(219,8).c_str());   // X pixel corrected
    yg = std::atof(line.substr(227,8).c_str());   // Y pixel corrected
    chi = std::atof(line.substr(235,8).c_str());   // Chi degrees
    other = std::atof(line.substr(243,8).c_str());   // Phi or Omega (non-scan) degrees

    rotRange.update(rot);
    rotrelRange.update(rotrel);

    // Check for discontinuities in goniostat angles, crystal number etc
    if (first) {
      // First
      SR.init(iaxis, chi, other, icryst, swing);
      first = false;
    } else {
      if (SR.NotSameRun(iaxis, chi, other, icryst, swing)) {
	//      if (chi != chiL || other != otherL
	//	  || icryst != icrystL || swing != swingL) {
	std::cout << "Change of run\n";
      }
    }

    chiL = chi;
    otherL = other;
    icrystL = icryst;
    swingL = swing;

    SR.StoreReflection(IVect3(h,k,l), s0r, s2r, rot, istl);

    if (iaxis == 2) {
      // Omega scan
      omega = rot;
      phi = other;
    } else {
      // Phi scan
      phi = rot;
      omega = other;
    }
    omegar = clipper::Util::d2rad(omega);
    chir = clipper::Util::d2rad(chi);
    phir = clipper::Util::d2rad(phi);

    // inverse rotation matrix R^-1
    Rmat = SaintRun::Rmatrix(omegar,chir,phir);
    RotInv = Rmat.inverse();
    // Back rotated s0x (zero angle frame) =  R^-1 s0
    Vec3<double> s0x = RotInv * SaintRun::s0v();

    int SAMPLE = 1;
    if (i%SAMPLE == 0) {
    ///    if (rot >= rot1 && rot <= rot2) {
      //      s0x = [U][K] s0r
      DVect3 hr(h,k,l);
      hrv.push_back(hr);
      s0xv.push_back(s0x);
      s0rv.push_back(s0r);
      Rv.push_back(Rmat);
      rotv.push_back(rot);

      // s2r is in reciprocal axis frame
      // Rotate into crystal frame 
      DVect3 ksr = Kmat * s0r;
      //  then  s0x = [U][K] s0r: accumulate lists
      s0rxf.push_back(clipper::Coord_orth(ksr));
      s0brt.push_back(clipper::Coord_orth(s0x));

      s2rv.push_back(s2r);
      // sr in direction of s2r + s0r
      double dstar = double(istl)/5000.;
      DVect3 sr = s2r + s0r;
      srv.push_back(dstar*sr.unit());
      //      srv.push_back(sr);
    }
    i++;
   }

  std::cout << "\n\nNumber = " << i << "\n";
  std::cout << "Size " << s0rxf.size() << " " << s0brt.size() << "\n";
  std::cout << "Detector swing " << swing << "\n";
  if (iaxis == 2) {
    std::cout << "Omega scan\n";
    std::cout << "Chi " <<  chi << "  Phi " << other << "\n\n";
  } else {
    std::cout << "Phi scan\n";
    std::cout << "Chi " <<  chi << "  Omega " << other << "\n\n";
  }

  std::cout
    << "Rotation range: absolute " << rotRange.min() << " - " << rotRange.max() << "\n"
    << "Rotation range: relative " << rotrelRange.min() << " - " << rotrelRange.max() << "\n\n";

  clipper::RTop_orth U(s0rxf, s0brt);
  std::cout << "[U] from [K]s0r & s0x = [R^-1]s0\n"
	    << U.format() << "\n"
	    << "Det = " <<  U.rot().det() << "\n";
  //  double EU = 0.0;
  //  for (int i=0;i<s0rxf.size();++i) {
  //    DVect3 s0rU = U.rot()*s0rxf[i];
  //    std::cout << "[U]s0r " << s0rU.format() << " [R^-1]s0 " << s0brt[i].format() << "\n";
  //  }


  /// --- get orientation stuff from s0r & s0x vectors
  SR.CalcResults();
  CMtz::MTZBAT BH = SR.MakeBatch(1, wvl);

  CMtz::MtzPrintBatchHeader(&BH);

  double E;
  DMat33 UK = MVutil::LsqTransform(s0rv, s0xv, E, true);
  std::cout << "\n[UK] from s0r/s0x Residual " << E << " for "
	    << s0rv.size() << " observations\n\n[UK] = \n"
	    << UK.format() << "\n\n"
	    << "Det[UK] = " << UK.det() << "\n";

  DMat33 KTK = UK.transpose()*UK;
  std::cout << "\n[K]T[K]\n" << KTK.format()  << "\n";

  //  MetricTensor KKM(KTK);
  //  Scell Kcell(KKM, false);
  Scell Kcell(UK.transpose()*UK, false);

  DMat33 sK =  Kcell.Bmat();
  DMat33 sU = UK * sK.inverse();
  std::cout << "\n[K] from KTK via metric tensor\n"
	    << sK.format() << "\n";
  std::cout <<"Equivalent cell: " << Kcell.format() << "\n";
  std::cout <<"\n[U] = [UK] [K^-1]\n"
	    << sU.format() << "\n" <<"Det = " << sU.det() <<"\n\n";


  DMat33 sC = MVutil::LsqTransform(hrv, srv, E);
  std::cout << "[C] from sr = [C]h"
    << "\nResidual " << E << " for "
	    << hrv.size() << " observations\n\n[C] = \n"
	    << sC.format() << "\n\n";
  std::cout <<"\n1/a*,b*c*  " << 1./sC(0,0) << " "<< 1./sC(1,1) << " "<< 1./sC(2,2) << "\n";

  // split in half
  std::vector<DVect3> srv2;
  std::vector<DVect3> hrv2;
  int n2 = srv.size()/2;

  hrv2.assign(hrv.begin(),hrv.begin()+n2);
  srv2.assign(srv.begin(),srv.begin()+n2);
  DMat33 C1 = MVutil::LsqTransform(hrv2, srv2, E);
  hrv2.assign(hrv.begin()+n2,hrv.end());
  srv2.assign(srv.begin()+n2,srv.end());
  double E2;
  DMat33 C2 = MVutil::LsqTransform(hrv2, srv2, E2);

  std::cout << "[C1]"     << "      Residual " << E << "\n"
	    << C1.format() << "\n\n";
  std::cout << "[C2]"     << "      Residual " << E2 << "\n"
	    << C2.format() << "\n\n";

  std::cout << "C1^-1 C2\n" << (C1.inverse() * C2).format() << "\n";


  // [B] = [K][C]
  // reciprocal metric tensor M = [B]T [B]
  DMat33 sB = sK * sC;
  Scell ncell(sB.transpose()*sB, false);
  std::cout <<"\nCell from [K][C] = "<<ncell.format()<<"\n\n";
  // [UB]
  DMat33 sUB = sU * sB;
  std::cout << "[UB] from [U] * [B]\n"<<sUB.format()<<"\n\n";

  Vec3<double> v0;
  v0[0]=0.; v0[1]=0.; v0[2]=0.;

  
  MGvector Vs02("s0s2r.vector", "s0s2r","black");
  Vs02.Vectors2(s0rv,s2rv, v0);

  MGvector Vs("sr.vector", "sr","red");
  Vs.Vectors(srv, v0);


  std::vector<DVect3> s2rdv;
  for (int i=0;i<s2rv.size();++i){
    // [R] [U] [K] s2r
    DVect3 s2rd = Rv[i] * UK * s2rv[i];
    s2rdv.push_back(s2rd);
  }
  MGvector Vs2("s2d.vector", "s2d","green");
  Vs2.Vectors(s2rdv, v0);

  if (p4pname != "null") {
    // Bruker convention
    DVect3 be1(0., 0., 1.0);  // Omega along +z
    DVect3 be2(-1.0, 0.0, 0.);  // Chi along -x
    DVect3 be3(0., 0., -1.0);  // Phi along -z


    //  sz = [UB]h
    std::vector<DVect3> szv;
    DMat33 bUK = bU * bK;
    // Only useful for omega scan
    DMat33 ChiPhi = clipper::Rotation(clipper::Euler_explicit
				      (be1, 0.0,
				       be2, chir,
				       be3, phir)).matrix();
    DMat33 ChPhUB = ChiPhi * bUB;
  }
  DVect3 s0 = SaintRun::s0v();
  std::cout << "s0 " << s0.format() << "\n";
  MeanSD rms;
  double maxd = 0.0;
  DMat33 bFi = bK.transpose().inverse();
  DMat33 bUFi = bU * bFi;
  std::cout << "b[U]\n" << bU.format() << "\n";
  std::cout << "b[F]^-1\n" << bFi.format() << "\n";
  std::cout << "b [F]^-1T[F]^-1\n" << (bFi.transpose()*bFi).format() <<"\n";
  std::cout << "b [U] [F^-1]\n" << bUFi.format()
	    <<"\nDet " << bUFi.det() <<"\n";

  std::vector<DVect3> s0rcv;
  std::vector<DVect3> s0zv;
  std::vector<DVect3> s0xcv;

  for (int i=0;i<s0rv.size();++i){
    // s0r calc = ([R] [UF^-1])^-1 s0
    //    DVect3 s0rc = (Rv[i] * bUFi).inverse() * s0;
    //    s0rcv.push_back(s0rc);
    DVect3 s0z = Rv[i].inverse() * s0;
    s0zv.push_back(s0z);

    DVect3 s0x = bU.inverse()* Rv[i].inverse() * s0;
    //    std::cout <<"s0x, s0r "<< s0x.format() <<" "<<s0rv[i].format()<<"\n";
    s0xcv.push_back(s0x);
    std::cout << rotv[i] << " " << Vlength(s0x)/Vlength(s0rv[i]) << "\n";
  }

  //  MGvector Vs0("s0rc.vector", "s0rc","red");
  //  Vs0.Vectors(s0rcv, v0);
  MGvector Vs0z("s0zc.vector", "s0zc","cyan");
  Vs0z.Vectors(s0zv, v0);

  MGvector Vs0x("s0xc.vector", "s0xc","blue");
  Vs0x.Vectors(s0xcv, v0);


  std::vector<DVect3> s0rr;

  for (int i=0;i<hrv.size();++i){  
    //
    //  s = [R] [U] [K] [C] h
    //  l      z   x   r         frames
    //  
    // we have from the data itself
    //   sUB   [UB]
    //   sK    [K]
    //   sC    [C] = diag(a*,b*,c*)
    //   sB    = [K][C]
    // for each reflection i we have
    //   hrv[i]   hkl
    //   s0r[i]   s0 in rec axis frame
    //   s2r[i]   s2 in rec axis frame
    //   Rv[i]    goniostat matrix [R]      
    DVect3 sz = sUB*hrv[i];
    DVect3 s = Rv[i] * sz;   // diffractometer frame, my conventions
    //    DVect3 s2l = -s0 + wvl*s; // s2 in rlu
    //    double s2len = sqrt(s2l*s2l); // should == 1
    //    maxd = Max(maxd, std::abs(s2len-1.0));
    //    rms.Add(s2len-1.0);

    // Info from p4p file
    DVect3 bs = Rv[i] * bUB *hrv[i];
    DVect3 s2l = -s0 + wvl*bs; // s2 in rlu
    double s2len = sqrt(s2l*s2l); // should == 1
    maxd = Max(maxd, std::abs(s2len-1.0));
    rms.Add(s2len-1.0);

    DVect3 s00 = Rv[i].inverse() * s0;
    s0rr.push_back(s00);

    if (i < 50) {
      DVect3 bs0 = (Rv[i] * bUFi) * s0rv[i];
      std::cout <<"bs0 " << bs0.format() <<"\n";

      DVect3 s00r =  bUFi * s0rv[i];
      std::cout << " s00, s00r " << s00.format() << " " << s00r.format() <<"\n";


    }
    //    std::cout << " bs " << bs.format()
    //          << "\n     bs0r " << bs0r.format() << " s0r " << s0rv[i].format() 
    //          << "\n     bs2r " << bs2r.format() << " s2r " << s2rv[i].format()
    //	      <<"\n";
    //      DVect3 bsz = bUB*hrv[i];
    //      DVect3 sz = ChPhUB*hrv[i];
    //      szv.push_back(sz);
  }

  std::cout <<"\n\n---- Maximum deviation of s/L radius from 1 = " << maxd
	    << " mean " << rms.Mean()
	    << " SD " << rms.SD()
	    << " N " << rms.Count() << "\n";

  double qE;
  DMat33 qUFi = MVutil::LsqTransform(s0rv, s0rr, qE, true);

  std::cout << "[UFi]"     << "      Residual " << qE << "\n"
	    << qUFi.format() << "\n\n";


}
